// src/backend/resource_policy.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef RESOURCE_POLICY_HH
#define RESOURCE_POLICY_HH 1

#include "config.h"
#include "gettext.h"

#include "glibmm/ustring.h"

#include <sgpemv2/policy_parameters.hh>
#include <sgpemv2/user_interrupt_exception.hh>
#include <sgpemv2/environment.hh>
#include <sgpemv2/sub_request.hh>

namespace sgpem
{
  class ResourcePolicy;

  /** \brief
  It's a Strategy wich stay for a resource allocating algorithm.
  It implements the related resource allocation policy.
  */
  class SG_DLLEXPORT ResourcePolicy
  {
  public:
    virtual ~ResourcePolicy();

    /**
      Initialize the inner components of the policy.

      Because it's a pure virtual method, must be re-implemented 
      in concrete derived classes.
     */
    virtual void configure() throw(UserInterruptException) = 0;

    /**
      Allocate resources to the threads 
     
      Because it's a pure virtual method, must be re-implemented 
      in concrete derived classes.
     */
    virtual void enforce(Environment& environment, Environment::SubRequestQueue& queue, SubRequest& sr) throw(UserInterruptException) = 0;

    /**
      Gets a string description of the policy.

      Because it's a pure virtual method, must be re-implemented 
      in concrete derived classes.
      \return String description of the policy.
     */
    virtual Glib::ustring get_description() const = 0;

    virtual Glib::ustring get_name() const = 0;

    virtual void activate() = 0;

    virtual void deactivate() = 0;

    /**
      Gets the parameters related with this policy.

      \return The policy parameters.
     */
    PolicyParameters& get_parameters();

  protected:
    PolicyParameters _parameters;
  };

}//~ namespace sgpem


#endif
